#!/usr/bin/env python

"""
Common handler functionality for different entities.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.data import get_address, get_uri, make_freebusy, to_part
from imiptools.dates import format_datetime
from imiptools.period import Period

class CommonFreebusy:

    "Common free/busy mix-in."

    def request(self):

        """
        Respond to a request by preparing a reply containing free/busy
        information for each indicated attendee.
        """

        oa = self.require_organiser_and_attendees()
        if not oa:
            return

        (organiser, organiser_attr), attendees = oa

        # Get the details for each attendee.

        responses = []
        rwrite = responses.append

        # For replies, the organiser and attendee are preserved.

        for attendee, attendee_attr in attendees.items():
            freebusy = self.store.get_freebusy(attendee)

            # Indicate the actual sender of the reply.

            if self.messenger:
                attendee_attr["SENT-BY"] = get_uri(self.messenger.sender)

            tzid = self.obj.get_tzid() or self.get_tzid()
            dtstart = self.obj.get_datetime("DTSTART")
            dtend = self.obj.get_datetime("DTEND")
            period = dtstart and dtend and Period(dtstart, dtend, tzid) or None

            rwrite(make_freebusy(freebusy, self.uid, organiser, organiser_attr, attendee, attendee_attr, period))

        # Return the reply.

        self.add_result("REPLY", [get_address(organiser)], to_part("REPLY", responses))

# vim: tabstop=4 expandtab shiftwidth=4
